/*
 *  Multiplucker.cpp
 *  everylayer2
 *
 *  Created by Andrew on 12/8/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "Multipluck.h"

Multipluck::Multipluck(StkFloat lowestFrequency)
: m_synths(new std::vector<Plucked *>), m_currentSynth(0), m_numSynths(8) {
	for (unsigned int i=0; i<m_numSynths; i++) {
		m_synths->push_back(new Plucked(lowestFrequency));
	}
}

Multipluck::~Multipluck() {
	if (m_synths) {
		for (unsigned int i=0; i<8; i++) {
			delete m_synths->at(i);
			m_synths->at(i) = NULL;
		}
		
		delete m_synths;
		m_synths = NULL;
	}
}

//! Start a note with the given frequency and amplitude.
void Multipluck::noteOn( StkFloat frequency, StkFloat amplitude ) {
	// start the note with the last-used synth
	m_synths->at(m_currentSynth)->noteOn(frequency, amplitude);
	
	// increment m_currentSynth
	m_currentSynth++;
	while (m_currentSynth >= m_numSynths) {
		m_currentSynth -= m_numSynths;
	}
}

//! Compute and return one output sample.
StkFloat Multipluck::tick(unsigned int channel) {
	StkFloat result = 0.0;
	
	for (unsigned int i=0; i<m_numSynths; i++) {
		result += m_synths->at(i)->tick();
	}
	
	return result;
}
