/*
 *  Renderer.cpp
 *  everylayer
 *
 *  Created by Andrew on 11/28/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "Renderer.h"

Renderer::Renderer() {
	m_children = new vector<Renderer *>;
	
    m_xPosition = new Vector3D(0.0f, 0.0f, 3.0f);
    m_yPosition = new Vector3D(0.0f, 0.0f, 3.0f);
    m_zPosition = new Vector3D(0.0f, 0.0f, 3.0f);
	
    m_xScale = new Vector3D(1.0f, 1.0f, 1.0f);
    m_yScale = new Vector3D(1.0f, 1.0f, 1.0f);
    m_zScale = new Vector3D(1.0f, 1.0f, 1.0f);
	
    m_rColor = new Vector3D(1.0f, 1.0f, 1.0f);
    m_gColor = new Vector3D(1.0f, 1.0f, 1.0f);
    m_bColor = new Vector3D(1.0f, 1.0f, 1.0f);
    m_aColor = new Vector3D(1.0f, 1.0f, 1.0f);
}

Renderer::~Renderer() {
	if (m_children) {
		// delete all children
		for (vector<Renderer *>::iterator it = m_children->begin();
			 it != m_children->end();
			 it++) {
			
			if (*it) {
				delete *it;
				*it = NULL;
			}
		}
		
		delete m_children;
		m_children = NULL;
	}
	
    if (m_xPosition) { delete m_xPosition; m_xPosition = NULL; }
    if (m_yPosition) { delete m_yPosition; m_yPosition = NULL; }
    if (m_zPosition) { delete m_zPosition; m_zPosition = NULL; }
	
    if (m_xScale) { delete m_xScale; m_xScale = NULL; }
    if (m_yScale) { delete m_yScale; m_yScale = NULL; }
    if (m_zScale) { delete m_zScale; m_zScale = NULL; }
	
    if (m_rColor) { delete m_rColor; m_rColor = NULL; }
    if (m_gColor) { delete m_gColor; m_gColor = NULL; }
    if (m_bColor) { delete m_bColor; m_bColor = NULL; }
    if (m_aColor) { delete m_aColor; m_aColor = NULL; }
}

void Renderer::render() {
	pushProperties();
	renderSelf();
	renderChildren();
	popProperties();
}

void Renderer::pushProperties() {
	glPushMatrix();
	glPushAttrib(GL_CURRENT_BIT);
	
	// scale, translate, set color
	glTranslatef(m_xPosition->value, m_yPosition->value, m_zPosition->value);
	glScalef(m_xScale->value, m_yScale->value, m_zScale->value);
	// glRotatef would go here
	
	// get current alpha and then push AlphaCurrent * m_aColor
	GLfloat color[4] = {0.0f, 0.0f, 0.0f, 0.0f};
	glGetFloatv(GL_CURRENT_COLOR, color);
	glColor4f(m_rColor->value, m_gColor->value, m_bColor->value, color[3] * m_aColor->value);
}

void Renderer::popProperties() {
	glPopAttrib();
	glPopMatrix();
}

void Renderer::renderSelf() {
	// subclasses override this method to do their drawing
}

void Renderer::renderChildren() {
	for (vector<Renderer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		
		(*it)->render();
	}
}

void Renderer::addChild(Renderer *child) {
	m_children->push_back(child);
}

void Renderer::removeChild(Renderer *child) {
	for (vector<Renderer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		
		if (*it == child) {
			delete *it;
			m_children->erase(it);
			break;
		}
		
	}
}

void Renderer::removeAllChildren() {
	for (vector<Renderer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		delete *it;
		*it = NULL;
	}
	m_children->clear();
}

Renderer * Renderer::childAt(unsigned int i) {
	return m_children->at(i);
}

unsigned int Renderer::numChildren() {
	return m_children->size();
}

void Renderer::setPosition(GLfloat x, GLfloat y, GLfloat z) {
	m_xPosition->update(x);
	m_yPosition->update(y);
	m_zPosition->update(z);
}

void Renderer::setScale(GLfloat x, GLfloat y, GLfloat z) {
	m_xScale->update(x);
	m_yScale->update(y);
	m_zScale->update(z);
}

void Renderer::setColor(GLfloat r, GLfloat g, GLfloat b) {
	m_rColor->update(r);
	m_gColor->update(g);
	m_bColor->update(b);
}

void Renderer::setAlpha(GLfloat a) {
	m_aColor->update(a);
}

void Renderer::interpolate(GLfloat delta) {
	for (vector<Renderer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		
		(*it)->interpolate(delta);
	}
	
    m_xPosition->interp(delta);
    m_yPosition->interp(delta);
    m_zPosition->interp(delta);
	
    m_xScale->interp(delta);
    m_yScale->interp(delta);
    m_zScale->interp(delta);
	
    m_rColor->interp(delta);
    m_gColor->interp(delta);
    m_bColor->interp(delta);
    m_aColor->interp(delta);
}
