/*
 *  Synthesizer.cpp
 *  everylayer
 *
 *  Created by Andrew on 11/28/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "Synthesizer.h"

Synthesizer::Synthesizer() : m_children( new vector<Synthesizer *> ) {
	pthread_mutex_init(&m_childrenMutex, NULL);
}

Synthesizer::~Synthesizer() {
	if (m_children) {
		// delete all children
		for (vector<Synthesizer *>::iterator it = m_children->begin();
			 it != m_children->end();
			 it++) {
			
			if (*it) {
				delete *it;
				*it = NULL;
			}
			
		}
		
		delete m_children;
		m_children = NULL;
	}
	
	pthread_mutex_destroy(&m_childrenMutex);
}

void Synthesizer::addChild(Synthesizer *child) {
	pthread_mutex_lock(&m_childrenMutex);
	
	m_children->push_back(child);
	
	pthread_mutex_unlock(&m_childrenMutex);
}

void Synthesizer::removeChild(Synthesizer *child) {
	pthread_mutex_lock(&m_childrenMutex);
	
	for (vector<Synthesizer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		
		if (*it == child) {
			delete *it;
			m_children->erase(it);
			break;
		}
		
	}

	pthread_mutex_unlock(&m_childrenMutex);
}

void Synthesizer::removeAllChildren() {
	pthread_mutex_lock(&m_childrenMutex);
	
	for (vector<Synthesizer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		delete *it;
		*it = NULL;
	}
	m_children->clear();
	
	pthread_mutex_unlock(&m_childrenMutex);
}

Synthesizer * Synthesizer::childAt(unsigned int i) {
	return m_children->at(i);
}

unsigned int Synthesizer::numChildren() {
	return m_children->size();
}

stk::StkFloat Synthesizer::tick(stk::StkFloat inputSample) {
	
	stk::StkFloat outputSample = 0.0;
	
	outputSample += synthesizeSelf(inputSample);
	outputSample += synthesizeChildren(inputSample);
	
	return outputSample;
}

stk::StkFloat Synthesizer::synthesizeSelf(stk::StkFloat inputSample) {
	return 0.0;
}

stk::StkFloat Synthesizer::synthesizeChildren(stk::StkFloat inputSample) {
	pthread_mutex_lock(&m_childrenMutex);
	
	stk::StkFloat outputSample = 0.0;
	
	// synthesize children
	for (vector<Synthesizer *>::iterator it = m_children->begin();
		 it != m_children->end();
		 it++) {
		
		outputSample += (*it)->tick(inputSample);
	}
	
	pthread_mutex_unlock(&m_childrenMutex);
	
	return outputSample;
}
