/*
 *  Track.cpp
 *  everylayer
 *
 *  Created by Andrew on 11/28/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "Track.h"

#include <sstream>

Track::Track() : m_duration(0), m_instrumentID(0), m_nextSample(0), m_notes(new vector<Note *>), m_selected(false) {
	pthread_mutex_init(&m_notesMutex, NULL);
}

Track::Track(unsigned int duration, unsigned int instrumentID)
: m_duration(duration), m_instrumentID(instrumentID), m_nextSample(0), m_notes(new vector<Note *>), m_selected(false) {
	pthread_mutex_init(&m_notesMutex, NULL);
}

Track::~Track() {
	if (m_notes) {
		for (unsigned int i=0; i<m_notes->size(); i++) {
			delete m_notes->at(i);
		}
		
		delete m_notes;
		m_notes = NULL;
	}
	
	pthread_mutex_destroy(&m_notesMutex);
}

unsigned int Track::nextSample() {
	return m_nextSample;
}

void Track::setNextSample(unsigned int n) {
	// increment the counter
	m_nextSample = n;
	while (m_nextSample >= m_duration)
	{
		m_nextSample -= m_duration;
	}
}

void Track::advanceNextSample() {
	// increment the counter
	m_nextSample++;
	while (m_nextSample >= m_duration)
	{
		m_nextSample -= m_duration;
	}
}

unsigned int Track::duration() {
	return m_duration;
}

unsigned int Track::instrumentID() {
	return m_instrumentID;
}

void Track::setInstrumentID(unsigned int instrumentID) {
	m_instrumentID = instrumentID;
}

void Track::addNote(unsigned int startTime, unsigned int stopTime, stk::StkFloat frequency, stk::StkFloat amplitude) {
	
	while (startTime >= m_duration) {
		startTime -= m_duration;
	}
	
	while (stopTime >= m_duration) {
		stopTime -= m_duration;
	}
	
	Note *n = new Note(startTime, stopTime, frequency, amplitude);
	
	pthread_mutex_lock(&m_notesMutex);
	
	m_notes->push_back(n);
	
	pthread_mutex_unlock(&m_notesMutex);
}

void Track::removeNote(Note *note) {
	pthread_mutex_lock(&m_notesMutex);
	
	for (vector<Note *>::iterator it = m_notes->begin();
		 it != m_notes->end();
		 it++) {
		
		if (*it == note) {
			delete *it;
			m_notes->erase(it);
			break;
		}
		
	}
	
	pthread_mutex_unlock(&m_notesMutex);
}

Note * Track::noteAt(unsigned int i) {
	return m_notes->at(i);
}

unsigned int Track::numNotes() {
	return m_notes->size();
}
