/*
 *  TrackRenderer.cpp
 *  everylayer
 *
 *  Created by Andrew on 11/28/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "TrackRenderer.h"

#include "Note.h"
#include <vector>

using namespace std;

TrackRenderer::TrackRenderer(Track *track)
:	m_track(track),
	m_width(new Vector3D(1.0, 1.0, 1.0)),
	m_height(new Vector3D(1.0, 1.0, 1.0)),
	m_deselectedRColor(1.0f),
	m_deselectedGColor(1.0f),
	m_deselectedBColor(1.0f),
	m_deselectedAColor(1.0f),
	m_selectedRColor(1.0f),
	m_selectedGColor(1.0f),
	m_selectedBColor(1.0f),
	m_selectedAColor(1.0f)
{
	m_rColor->slew = 5.0;
	m_gColor->slew = 5.0;
	m_bColor->slew = 5.0;
	m_aColor->slew = 5.0;
}

TrackRenderer::~TrackRenderer() {
    if (m_width) { delete m_width; m_width = NULL; }
    if (m_height) { delete m_height; m_height = NULL; }
}

Track * TrackRenderer::track() {
	return m_track;
}

void TrackRenderer::renderSelf() {
	// Render each note
	
	// go into track coordinates (spread over the desired width)
	glPushMatrix();
	glScalef(m_width->value/(GLfloat)m_track->duration(), 1.0f, 1.0f);
	
	glPushAttrib(GL_CURRENT_BIT);
	GLfloat color[4] = {0.0f, 0.0f, 0.0f, 0.0f};
	glGetFloatv(GL_CURRENT_COLOR, color);
	
	for (unsigned int i=0; i<m_track->numNotes(); i++) {
		Note *note = m_track->noteAt(i);
		
		GLfloat alpha = max( 0.1, 1.0 - (GLfloat)(m_track->nextSample() - note->startTime()) / 30000.0);
		glColor4f(m_rColor->value, m_gColor->value, m_bColor->value, color[3] * alpha);
		glBegin(GL_QUADS);
		glVertex2f(note->startTime(),	0);
		glVertex2f(note->stopTime(),	0);
		glVertex2f(note->stopTime(),	m_height->value);
		glVertex2f(note->startTime(),	m_height->value);
		glEnd();
	}
	
	glPopAttrib();
	glPopMatrix();
	
	
	// Render the playhead
	
	glPushAttrib(GL_CURRENT_BIT);
	
	glColor3f(1.0f, 0.0f, 0.0f);
	
	GLfloat x = m_width->value * (GLfloat)m_track->nextSample() / (GLfloat)m_track->duration();
	GLfloat w = 4.0f;
	
	glBegin(GL_QUADS);
	glVertex2f(x,	0);
	glVertex2f(x+w,	0);
	glVertex2f(x+w,	m_height->value);
	glVertex2f(x,	m_height->value);
	glEnd();
	
	glPopAttrib();
}

void TrackRenderer::setWidth(GLfloat w) {
	m_width->update(w);
}

void TrackRenderer::setHeight(GLfloat h) {
	m_height->update(h);
}

void TrackRenderer::setColor(GLfloat r, GLfloat g, GLfloat b) {
	m_deselectedRColor = r;
	m_deselectedGColor = g;
	m_deselectedBColor = b;
}

void TrackRenderer::setAlpha(GLfloat a) {
	m_deselectedAColor = a;
}

void TrackRenderer::setSelectedColor(GLfloat r, GLfloat g, GLfloat b) {
	m_selectedRColor = r;
	m_selectedGColor = g;
	m_selectedBColor = b;
}

void TrackRenderer::setSelectedAlpha(GLfloat a) {
	m_selectedAColor = a;
}

void TrackRenderer::interpolate(GLfloat delta) {
	if (m_track->selected()) {
		Renderer::setColor(m_selectedRColor, m_selectedGColor, m_selectedBColor);
		Renderer::setAlpha(m_selectedAColor);
	} else {
		Renderer::setColor(m_deselectedRColor, m_deselectedGColor, m_deselectedBColor);
		Renderer::setAlpha(m_deselectedAColor);
	}
	
	Renderer::interpolate(delta);
	
	m_width->interp(delta);
    m_height->interp(delta);
}
