/*
 *  TrackSynthesizer.cpp
 *  everylayer
 *
 *  Created by Andrew on 11/28/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "TrackSynthesizer.h"

#include <vector>

#include "stk/Drummer.h"
#include "stk/BeeThree.h"
#include "stk/HevyMetl.h"
#include "stk/Plucked.h"
#include "stk/Sitar.h"

#import "Multipluck.h"

#define NUM_INSTRUMENTS 2

TrackSynthesizer::TrackSynthesizer(Track *track) {
	m_track = track;
	
	m_instruments = new vector<stk::Instrmnt *>;
	for (unsigned int i=0; i<NUM_INSTRUMENTS; i++) {
		m_instruments->push_back(NULL);
	}
}

TrackSynthesizer::~TrackSynthesizer() {
	if (m_instruments) {
		for (unsigned int i=0; i<m_instruments->size(); i++) {
			if (m_instruments->at(i)) {
				delete m_instruments->at(i);
				m_instruments->at(i) = NULL;
			}
		}
		
		delete m_instruments;
		m_instruments = NULL;
	}
}

Track * TrackSynthesizer::track() {
	return m_track;
}

stk::StkFloat TrackSynthesizer::synthesizeSelf(stk::StkFloat inputSample) {
	
	stk::StkFloat outputSample = 0.0;
	
	// do synthesis here
	stk::Instrmnt *synth = instrument(m_track->instrumentID());
	unsigned int nextSample = m_track->nextSample();
	
	if (synth) {
		for (unsigned int i=0; i<m_track->numNotes(); i++) {
			Note * note = m_track->noteAt(i);
			if (note->startTime() == nextSample) {
				synth->noteOn(note->frequency(), note->amplitude());
			}/*else if (note->stopTime() == nextSample) {
				synth->noteOff(10000.0);
			}*/
		}
	}
	
	outputSample = synth->tick();
	
	m_track->advanceNextSample();
	
	return outputSample;
}

stk::Instrmnt * TrackSynthesizer::instrument(unsigned int instrumentID) {
	stk::Instrmnt *instrument = NULL;
	
	if (instrumentID < m_instruments->size()) {
		instrument = m_instruments->at(instrumentID);
		
		if (instrument == NULL) {
			switch (instrumentID) {
				case 1: instrument = new stk::Drummer(); break;
				//case 1: instrument = new stk::BeeThree(); break;
				//case 2: instrument = new stk::HevyMetl(); break;
				case 0: instrument = new Multipluck(110.0); break;
				//case 4: instrument = new stk::Sitar(); break;
				default: break;
			}
			m_instruments->at(instrumentID) = instrument;
		}
	}
	
	return instrument;
}
